==> openturns_0.12.2.orig.tar.gz: openturns-0.12.2/lib/src/Uncertainty/Distribution/MultiNomialFactory.cxx <==
//                                               -*- C++ -*-
/**
 *  @file  MultiNomialFactory.cxx
 *  @brief Factory for MultiNomial distribution
 *
 *  (C) Copyright 2005-2007 EDF-EADS-Phimeca
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License.
 *
 *  This library is distributed in the hope that it will be useful
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
 *
 *  @author: $LastChangedBy: dutka $
 *  @date:   $LastChangedDate: 2007-05-10 16:43:31 +0200 (jeu 10 mai 2007) $
 *  Id:      $Id: MultiNomialFactory.cxx 434 2007-05-10 14:43:31Z dutka $
 */
#include <cmath>
#include "MultiNomialFactory.hxx"
#include "MultiNomial.hxx"

namespace OpenTURNS {

  namespace Uncertainty {

    namespace Distribution {

      /* Default constructor */
      MultiNomialFactory::MultiNomialFactory()
      {
	// Initialize any other class members here
	// At last, allocate memory space if needed, but go to destructor to free it
      }

      /* Virtual constructor */
      MultiNomialFactory * MultiNomialFactory::clone() const
      {
	return new MultiNomialFactory(*this);
      }


      /* Here is the interface that all derived class must implement */

      MultiNomialFactory::DistributionImplementation * MultiNomialFactory::buildImplementation(const NumericalSample & sample) const throw(InvalidArgumentException, InternalException)
      {
	if (sample.getSize() == 0) throw InvalidArgumentException(HERE) << "Error: cannot build a MultiNomial distribution from an empty sample";
	// Check if each component of the sample is an integer, compute the mean and extract the maximum value
	UnsignedLong size(sample.getSize());
	UnsignedLong dimension(sample.getDimension());
	NumericalPoint p(dimension, 0.0);
	NumericalScalar max(sample[0][0]);
	for (UnsignedLong i = 0; i < size; i++)
	  {
	    for (UnsignedLong j = 0; j < dimension; j++)
	      {
		NumericalScalar x(sample[i][j]);
		if ((x != trunc(x)) || (x < 0.0)) throw InternalException(HERE) << "Error: can build a MultiNomial distribution only from a sample with positive integer components, here sample[" << i << "][" << j << "]=" << x;
		if (x > max) max = x;
		p[j] += x;
	      }
	  }
	UnsignedLong n((UnsignedLong)max);
	p *= 1.0 / (max * size);
	return MultiNomial(p, n).clone();
      }


    } /* namespace Distribution */
  } /* namespace Uncertainty */
} /* namespace OpenTURNS */

==> openturns_0.13.1.orig.tar.gz: openturns-0.13.1/lib/src/Uncertainty/Distribution/MultiNomialFactory.cxx <==
//                                               -*- C++ -*-
/**
 *  @file  MultiNomialFactory.cxx
 *  @brief Factory for MultiNomial distribution
 *
 *  (C) Copyright 2005-2007 EDF-EADS-Phimeca
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License.
 *
 *  This library is distributed in the hope that it will be useful
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
 *
 *  @author: $LastChangedBy: dutka $
 *  @date:   $LastChangedDate: 2009-05-28 10:54:56 +0200 (jeu. 28 mai 2009) $
 *  Id:      $Id: MultiNomialFactory.cxx 1261 2009-05-28 08:54:56Z dutka $
 */
#include <cmath>
#include "MultiNomialFactory.hxx"
#include "MultiNomial.hxx"

namespace OpenTURNS {

  namespace Uncertainty {

    namespace Distribution {

      /* Default constructor */
      MultiNomialFactory::MultiNomialFactory()
      {
	// Initialize any other class members here
	// At last, allocate memory space if needed, but go to destructor to free it
      }

      /* Virtual constructor */
      MultiNomialFactory * MultiNomialFactory::clone() const
      {
	return new MultiNomialFactory(*this);
      }


      /* Here is the interface that all derived class must implement */

      MultiNomialFactory::DistributionImplementation * MultiNomialFactory::buildImplementation(const NumericalSample & sample) const throw(InvalidArgumentException, InternalException)
      {
	if (sample.getSize() == 0) throw InvalidArgumentException(HERE) << "Error: cannot build a MultiNomial distribution from an empty sample";
	// Check if each component of the sample is an integer, compute the mean and extract the maximum value
	UnsignedLong size(sample.getSize());
	UnsignedLong dimension(sample.getDimension());
	NumericalPoint p(dimension, 0.0);
	NumericalScalar max(sample[0][0]);
	for (UnsignedLong i = 0; i < size; i++)
	  {
	    for (UnsignedLong j = 0; j < dimension; j++)
	      {
		NumericalScalar x(sample[i][j]);
		if ((x != trunc(x)) || (x < 0.0)) throw InternalException(HERE) << "Error: can build a MultiNomial distribution only from a sample with positive integer components, here sample[" << i << "][" << j << "]=" << x;
		if (x > max) max = x;
		p[j] += x;
	      }
	  }
	UnsignedLong n((UnsignedLong)max);
	p *= 1.0 / (max * size);
	return MultiNomial(p, n).clone();
      }

      MultiNomialFactory::DistributionImplementation * MultiNomialFactory::buildImplementation() const throw(InvalidArgumentException, InternalException)
      {
	return MultiNomial().clone();
      }


    } /* namespace Distribution */
  } /* namespace Uncertainty */
} /* namespace OpenTURNS */
